package patched.ajax4jsf;

import java.util.Iterator;
import java.util.Set;

import javax.faces.component.ContextCallback;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;

import org.ajax4jsf.component.ContextCallbackWrapper;
import org.ajax4jsf.component.html.HtmlAjaxRegion;
import org.ajax4jsf.context.AjaxContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class PatchedHtmlAjaxRegion extends HtmlAjaxRegion {

    // Private Fields
    private static final Log log = LogFactory.getLog(PatchedHtmlAjaxRegion.class);

    private static final  ContextCallback decodeCallback= new ContextCallback(){
		public void invokeContextCallback(FacesContext context,
				UIComponent target) {
			target.processDecodes(context);
		}
	};

	private static final  ContextCallback validateCallback= new ContextCallback(){
		public void invokeContextCallback(FacesContext context,
				UIComponent target) {
			target.processValidators(context);
		}
	};

	private static final  ContextCallback updateCallback= new ContextCallback(){
		public void invokeContextCallback(FacesContext context,
				UIComponent target) {
			target.processUpdates(context);
		}
	};

	private void intelligentInvokeOnComponent(FacesContext context, String areaId, ContextCallbackWrapper contextCallbackWrapper) {
		boolean foundAsChild = invokeOnComponent(context, areaId, contextCallbackWrapper);
		if (foundAsChild) {
			return;
		}
		
		UIViewRoot viewRoot = getFacesContext().getViewRoot();

		boolean foundGlobally = viewRoot.invokeOnComponent(context, areaId, contextCallbackWrapper);
		if (foundGlobally) {
			return;
		}
		
		if (log.isDebugEnabled()) {
			AjaxContext ajaxContext = AjaxContext.getCurrentInstance(context);
			String ajaxSingleClientId = ajaxContext.getAjaxSingleClientId();
			log.warn("Unable to find component named '" + areaId + "' listed in process attribute for component '" + ajaxSingleClientId + "'.");
		}
	}
	
	@Override
	public void processDecodes(FacesContext context) {
		AjaxContext ajaxContext = AjaxContext.getCurrentInstance(context);
		String ajaxSingleClientId = ajaxContext.getAjaxSingleClientId();
		// Process this component itself
		try {
			decode(context);
		} catch (RuntimeException e) {
			context.renderResponse();
			throw e;
		}
		if (ajaxContext.isAjaxRequest() && null != ajaxSingleClientId) {
			invokeOnComponent(context, ajaxSingleClientId, new ContextCallbackWrapper(decodeCallback));
			Set<String> areasToProcess = ajaxContext.getAjaxAreasToProcess();
			if(null != areasToProcess){
				for (String areaId : areasToProcess) {
					intelligentInvokeOnComponent(context, areaId, new ContextCallbackWrapper(decodeCallback));
				}
			}
		} else {
			// Process all facets and children of this component
			Iterator<UIComponent> kids = getFacetsAndChildren();
			while (kids.hasNext()) {
				UIComponent kid = kids.next();
				kid.processDecodes(context);
			}
		}
	}

	@Override
	public void processValidators(FacesContext context) {
		AjaxContext ajaxContext = AjaxContext.getCurrentInstance(context);
		String ajaxSingleClientId = ajaxContext.getAjaxSingleClientId();
		if (ajaxContext.isAjaxRequest() && null != ajaxSingleClientId) {
			invokeOnComponent(context, ajaxSingleClientId,  new ContextCallbackWrapper(validateCallback));
			Set<String> areasToProcess = ajaxContext.getAjaxAreasToProcess();
			if(null != areasToProcess){
				for (String areaId : areasToProcess) {
					intelligentInvokeOnComponent(context, areaId, new ContextCallbackWrapper(validateCallback));
				}
			}
		} else {
			super.processValidators(context);
		}
	}

	@Override
	public void processUpdates(FacesContext context) {
		AjaxContext ajaxContext = AjaxContext.getCurrentInstance(context);
		String ajaxSingleClientId = ajaxContext.getAjaxSingleClientId();
		if (ajaxContext.isAjaxRequest() && null != ajaxSingleClientId) {
			invokeOnComponent(context, ajaxSingleClientId, new ContextCallbackWrapper(updateCallback));
			Set<String> areasToProcess = ajaxContext.getAjaxAreasToProcess();
			if(null != areasToProcess){
				for (String areaId : areasToProcess) {
					intelligentInvokeOnComponent(context, areaId, new ContextCallbackWrapper(updateCallback));
				}
			}
		} else {
			super.processUpdates(context);
		}
	}

}
